@@ -2,16 +2,22 @@ import errno
 from os import path
 from os import strerror as os_strerror
 
+try:
+    from collections.abc import Iterable
+    from collections.abc import Mapping
+except ImportError:
+    from collections import Iterable
+    from collections import Mapping
+
 import six
 from attrdict import AttrDict
 
-from qbittorrentapi.helpers import APINames
-from qbittorrentapi.helpers import ClientCache
-from qbittorrentapi.helpers import Dictionary
-from qbittorrentapi.helpers import is_version_less_than
-from qbittorrentapi.helpers import List
-from qbittorrentapi.helpers import ListEntry
-from qbittorrentapi.helpers import list2string
+from qbittorrentapi.definitions import APINames
+from qbittorrentapi.definitions import ClientCache
+from qbittorrentapi.definitions import Dictionary
+from qbittorrentapi.definitions import List
+from qbittorrentapi.definitions import ListEntry
+from qbittorrentapi.definitions import TorrentStates
 from qbittorrentapi.decorators import Alias
 from qbittorrentapi.decorators import aliased
 from qbittorrentapi.decorators import login_required
@@ -26,17 +32,16 @@ from qbittorrentapi.request import Request
 
 @aliased
 class TorrentDictionary(Dictionary):
-
     """
     Alows interaction with individual torrents via the "Torrents" API endpoints.
 
-    Usage:
+    :Usage:
         >>> from qbittorrentapi import Client
         >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
         >>> # this are all the same attributes that are available as named in the
         >>> #  endpoints or the more pythonic names in Client (with or without 'transfer_' prepended)
         >>> torrent = client.torrents.info()[0]
-        >>> hash = torrent.info.hash
+        >>> torrent_hash = torrent.info.hash
         >>> # Attributes without inputs and a return value are properties
         >>> properties = torrent.properties
         >>> trackers = torrent.trackers
@@ -61,9 +66,18 @@ class TorrentDictionary(Dictionary):
         for name, value in self.info.items():
             setattr(self, name, value)
 
+    @property
+    def state_enum(self):
+        """Returns the formalized Enumeration for Torrent State instead of the raw string."""
+        try:
+            return TorrentStates(self.state)
+        except ValueError:
+            return TorrentStates.UNKNOWN
+
     @property
     def info(self):
-        if is_version_less_than(self._client._app_web_api_version_from_version_checker(), '2.0.1', lteq=False):
+        api_version = self._client._app_web_api_version_from_version_checker()
+        if self._client._is_version_less_than(api_version, '2.0.1', lteq=False):
             info = [t for t in self._client.torrents_info() if t.hash == self._torrent_hash]
         else:
             info = self._client.torrents_info(torrent_hashes=self._torrent_hash)
@@ -300,11 +314,10 @@ class Tag(ListEntry):
 
 
 class Torrents(ClientCache):
-
     """
     Allows interaction with the "Torrents" API endpoints.
 
-    Usage:
+    :Usage:
         >>> from qbittorrentapi import Client
         >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
         >>> # this are all the same attributes that are available as named in the
@@ -437,11 +450,10 @@ class Torrents(ClientCache):
 
 @aliased
 class TorrentCategories(ClientCache):
-
     """
     Alows interaction with torrent categories within the "Torrents" API endpoints.
 
-    Usage:
+    :Usage:
         >>> from qbittorrentapi import Client
         >>> client = Client(host='localhost:8080', username='admin', password='adminadmin')
         >>> # this are all the same attributes that are available as named in the
@@ -483,7 +495,6 @@ class TorrentCategories(ClientCache):
 
 @aliased
 class TorrentTags(ClientCache):
-
     """
     Allows interaction with torrent tags within the "Torrent" API endpoints.
 
@@ -523,8 +534,7 @@ class TorrentTags(ClientCache):
 
 @aliased
 class TorrentsAPIMixIn(Request):
-
-    """Implementation of all Torrents API methods"""
+    """Implementation of all Torrents API methods."""
 
     @property
     def torrents(self):
@@ -571,13 +581,21 @@ class TorrentsAPIMixIn(Request):
         """
         Add one or more torrents by URLs and/or torrent files.
 
-        Exceptions:
-            UnsupportedMediaType415Error if file is not a valid torrent file
-            TorrentFileNotFoundError if a torrent file doesn't exist
-            TorrentFilePermissionError if read permission is denied to torrent file
+        :raises UnsupportedMediaType415Error: if file is not a valid torrent file
+        :raises TorrentFileNotFoundError: if a torrent file doesn't exist
+        :raises TorrentFilePermissionError: if read permission is denied to torrent file
 
-        :param urls: List of URLs (http://, https://, magnet: and bc://bt/)
-        :param torrent_files: list of torrent files
+        :param urls: single instance or an iterable of URLs (http://, https://, magnet: and bc://bt/)
+        :param torrent_files: several options are available to send torrent files to qBittorrent:
+                              1) single instance of bytes: useful if torrent file already read from disk or downloaded from internet.
+                              2) single instance of file handle to torrent file: use open(<filepath>, 'rb') to open the torrent file.
+                              3) single instance of a filepath to torrent file: e.g. '/home/user/torrent_filename.torrent'
+                              4) an iterable of the single instances above to send more than one torrent file
+                              5) dictionary with key/value pairs of torrent name and single instance of above object
+                              Note: The torrent name in a dictionary is useful to identify which torrent file
+                              errored. qBittorrent provides back that name in the error text. If a torrent
+                              name is not provided, then the name of the file will be used. And in the case of
+                              bytes (or if filename cannot be determined), the value 'torrent__n' will be used
         :param save_path: location to save the torrent data
         :param cookie: cookie to retrieve torrents by URL
         :param category: category to assign to torrent(s)
@@ -593,7 +611,7 @@ class TorrentsAPIMixIn(Request):
         :return: "Ok." for success and "Fails." for failure
         """
 
-        data = {'urls': (None, list2string(urls, '\n')),
+        data = {'urls': (None, self._list2string(urls, '\n')),
                 'savepath': (None, save_path),
                 'cookie': (None, cookie),
                 'category': (None, category),
@@ -607,24 +625,70 @@ class TorrentsAPIMixIn(Request):
                 'sequentialDownload': (None, is_sequential_download),
                 'firstLastPiecePrio': (None, is_first_last_piece_priority)}
 
-        files = None
-        if torrent_files:
-            if isinstance(torrent_files, six.string_types):
-                torrent_files = [torrent_files]
-            files = []
-            for torrent_file in torrent_files:
-                try:
-                    filename = path.abspath(path.realpath(path.expanduser(torrent_file)))
-                    files.append((path.basename(filename), open(filename, 'rb')))
-                except IOError as io_err:
-                    if io_err.errno == errno.ENOENT:
-                        raise TorrentFileNotFoundError(errno.ENOENT, os_strerror(errno.ENOENT), torrent_file)
-                    elif io_err.errno == errno.EACCES:
-                        raise TorrentFilePermissionError(errno.ENOENT, os_strerror(errno.EACCES), torrent_file)
-                    raise TorrentFileError(io_err)
+        files = self._normalize_torrent_files(torrent_files)
 
         return self._post(_name=APINames.Torrents, _method='add', data=data, files=files, **kwargs)
 
+    @staticmethod
+    def _normalize_torrent_files(user_files):
+        """
+        Normalize the torrent file(s) from the user.
+        The file(s) can be the raw bytes, file handle, filepath for a torrent file,
+        or a Sequence (e.g. list|set|tuple) of any of these "files".
+        Further, the file(s) can be in a dictionary with the "names" of the torrents as the keys.
+        These "names" can be anything...but are mostly useful as identifiers for each file.
+        """
+        if not user_files:
+            return None
+
+        prefix = 'torrent__'
+        # if it's string-like and not a list|set|tuple, then make it a list
+        # checking for 'read' attr since a single file handle is iterable but also needs to be in a list
+        if isinstance(user_files, six.string_types) or not isinstance(user_files, Iterable) or hasattr(user_files, 'read'):
+            user_files = [user_files]
+
+        # up convert to a dictionary to add fabricated torrent names
+        norm_files = user_files if isinstance(user_files, Mapping) else {prefix + str(i): f for i, f in enumerate(user_files)}
+
+        files = {}
+        for name, torrent_file in norm_files.items():
+            try:
+                fh = None
+                if isinstance(torrent_file, bytes):
+                    # since strings are bytes on python 2, simple filepaths will end up here
+                    # just check if it's a file first in that case...
+                    # this does prevent providing more useful IO errors on python 2....but it's dead anyway...
+                    try:
+                        filepath = path.abspath(path.realpath(path.expanduser(str(torrent_file))))
+                        if path.exists(filepath):
+                            fh = open(filepath, 'rb')
+                            name = path.basename(filepath)
+                    except Exception:
+                        fh = None
+                    # if bytes, assume it's a raw torrent file that was downloaded or read from disk
+                    if not fh:
+                        fh = torrent_file
+                elif hasattr(torrent_file, 'read') and callable(torrent_file.read):
+                    # if hasattr('read'), assume this is a file handle from open() or similar...
+                    #  there isn't a reliable way to detect a file-like object on both python 2 & 3
+                    fh = torrent_file
+                else:
+                    # otherwise, coerce to a string and try to open it as a file
+                    filepath = path.abspath(path.realpath(path.expanduser(str(torrent_file))))
+                    fh = open(filepath, 'rb')
+                    name = path.basename(filepath)
+
+                # if using default name, let Requests try to figure out the filename to send
+                # Requests will fall back to "name" as the dict key if fh doesn't provide a file name
+                files[name] = fh if name.startswith(prefix) else (name, fh)
+            except IOError as io_err:
+                if io_err.errno == errno.ENOENT:
+                    raise TorrentFileNotFoundError(errno.ENOENT, os_strerror(errno.ENOENT), torrent_file)
+                elif io_err.errno == errno.EACCES:
+                    raise TorrentFilePermissionError(errno.ENOENT, os_strerror(errno.EACCES), torrent_file)
+                raise TorrentFileError(io_err)
+        return files or None
+
     ##########################################################################
     # INDIVIDUAL TORRENT ENDPOINTS
     ##########################################################################
@@ -634,12 +698,11 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's properties.
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: Dictionary of torrent properties
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#get-torrent-generic-properties
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-generic-properties
         """
         data = {'hash': torrent_hash or kwargs.pop('hash')}
         return self._post(_name=APINames.Torrents, _method='properties', data=data, **kwargs)
@@ -650,12 +713,11 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's trackers.
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: List of torrent's trackers
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#get-torrent-trackers
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-trackers
         """
         data = {'hash': torrent_hash or kwargs.pop('hash')}
         return self._post(_name=APINames.Torrents, _method='trackers', data=data, **kwargs)
@@ -666,12 +728,11 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's web seeds.
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: List of torrent's web seeds
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#get-torrent-web-seeds
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-web-seeds
         """
         data = {'hash': torrent_hash or kwargs.pop('hash')}
         return self._post(_name=APINames.Torrents, _method='webseeds', data=data, **kwargs)
@@ -682,12 +743,11 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's files.
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: List of torrent's files
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#get-torrent-contents
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-contents
         """
         data = {'hash': torrent_hash or kwargs.pop('hash')}
         return self._post(_name=APINames.Torrents, _method='files', data=data, **kwargs)
@@ -699,8 +759,7 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's pieces' states. (alias: torrents_pieceStates)
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: list of torrent's pieces' states
@@ -715,8 +774,7 @@ class TorrentsAPIMixIn(Request):
         """
         Retrieve individual torrent's pieces' hashes. (alias: torrents_pieceHashes)
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :return: List of torrent's pieces' hashes
@@ -730,15 +788,14 @@ class TorrentsAPIMixIn(Request):
         """
         Add trackers to a torrent. (alias: torrents_addTrackers)
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :param urls: tracker urls to add to torrent
         :return: None
         """
         data = {'hash': torrent_hash or kwargs.pop('hash'),
-                'urls': list2string(urls, '\n')}
+                'urls': self._list2string(urls, '\n')}
         self._post(_name=APINames.Torrents, _method='addTrackers', data=data, **kwargs)
 
     @version_implemented('2.2.0', 'torrents/editTracker')
@@ -748,10 +805,9 @@ class TorrentsAPIMixIn(Request):
         """
         Replace a torrent's tracker with a different one. (alias: torrents_editTrackers)
 
-        Exceptions:
-            InvalidRequest400
-            NotFound404Error
-            Conflict409Error
+        :raises InvalidRequest400:
+        :raises NotFound404Error:
+        :raises Conflict409Error:
 
         :param torrent_hash: hash for torrent
         :param original_url: URL for existing tracker
@@ -770,16 +826,15 @@ class TorrentsAPIMixIn(Request):
         """
         Remove trackers from a torrent. (alias: torrents_removeTrackers)
 
-        Exceptions:
-            NotFound404Error
-            Conflict409Error
+        :raises NotFound404Error:
+        :raises Conflict409Error:
 
         :param torrent_hash: hash for torrent
         :param urls: tracker urls to removed from torrent
         :return: None
         """
         data = {'hash': torrent_hash or kwargs.pop('hash'),
-                'urls': list2string(urls, '|')}
+                'urls': self._list2string(urls, '|')}
         self._post(_name=APINames.Torrents, _method='removeTrackers', data=data, **kwargs)
 
     @Alias('torrents_filePrio')
@@ -788,18 +843,17 @@ class TorrentsAPIMixIn(Request):
         """
         Set priority for one or more files. (alias: torrents_filePrio)
 
-        Exceptions:
-            InvalidRequest400 if priority is invalid or at least one file ID is not an integer
-            NotFound404Error
-            Conflict409 if torrent metadata has not finished downloading or at least one file was not found
+        :raises InvalidRequest400: if priority is invalid or at least one file ID is not an integer
+        :raises NotFound404Error:
+        :raises Conflict409: if torrent metadata has not finished downloading or at least one file was not found
         :param torrent_hash: hash for torrent
-        :param file_ids: single file ID or a list. See
+        :param file_ids: single file ID or a list.
         :param priority: priority for file(s)
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#set-file-priority
-        :return:
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#set-file-priority
+        :return: None
         """
         data = {'hash': torrent_hash or kwargs.pop('hash'),
-                'id': list2string(file_ids, "|"),
+                'id': self._list2string(file_ids, "|"),
                 'priority': priority}
         self._post(_name=APINames.Torrents, _method='filePrio', data=data, **kwargs)
 
@@ -808,8 +862,7 @@ class TorrentsAPIMixIn(Request):
         """
         Rename a torrent.
 
-        Exceptions:
-            NotFound404Error
+        :raises NotFound404Error:
 
         :param torrent_hash: hash for torrent
         :param new_torrent_name: new name for torrent
@@ -826,10 +879,9 @@ class TorrentsAPIMixIn(Request):
         """
         Rename a torrent file.
 
-        Exceptions:
-            MissingRequiredParameters400Error
-            NotFound404Error
-            Conflict409Error
+        :raises MissingRequiredParameters400Error:
+        :raises NotFound404Error:
+        :raises Conflict409Error:
 
         :param torrent_hash: hash for torrent
         :param file_id: id for file
@@ -862,7 +914,7 @@ class TorrentsAPIMixIn(Request):
         :param offset: Start of list (if < 0, offset from end of list)
         :param torrent_hashes: Filter list by hash (separate multiple hashes with a '|')
         :return: List of torrents
-            Properties: https://github.com/qbittorrent/qBittorrent/wiki/Web-API-Documentation#get-torrent-list
+            Properties: https://github.com/qbittorrent/qBittorrent/wiki/WebUI-API-(qBittorrent-4.1)#get-torrent-list
         """
         data = {'filter': status_filter,
                 'category': category,
@@ -870,7 +922,7 @@ class TorrentsAPIMixIn(Request):
                 'reverse': reverse,
                 'limit': limit,
                 'offset': offset,
-                'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+                'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         return self._post(_name=APINames.Torrents, _method='info', data=data, **kwargs)
 
     @login_required
@@ -881,7 +933,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='resume', data=data, **kwargs)
 
     @login_required
@@ -892,7 +944,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), "|")}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), "|")}
         self._post(_name=APINames.Torrents, _method='pause', data=data, **kwargs)
 
     @login_required
@@ -904,7 +956,7 @@ class TorrentsAPIMixIn(Request):
         :param delete_files: True to delete the torrent's files
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'deleteFiles': delete_files}
         self._post(_name=APINames.Torrents, _method='delete', data=data, **kwargs)
 
@@ -916,7 +968,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='recheck', data=data, **kwargs)
 
     @version_implemented('2.0.2', 'torrents/reannounce')
@@ -930,7 +982,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='reannounce', data=data, **kwargs)
 
     @Alias('torrents_increasePrio')
@@ -939,13 +991,12 @@ class TorrentsAPIMixIn(Request):
         """
         Increase the priority of a torrent. Torrent Queuing must be enabled. (alias: torrents_increasePrio)
 
-        Exceptions:
-            Conflict409
+        :raises Conflict409:
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='increasePrio', data=data, **kwargs)
 
     @Alias('torrents_decreasePrio')
@@ -954,13 +1005,12 @@ class TorrentsAPIMixIn(Request):
         """
         Decrease the priority of a torrent. Torrent Queuing must be enabled. (alias: torrents_decreasePrio)
 
-        Exceptions:
-            Conflict409
+        :raises Conflict409:
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='decreasePrio', data=data, **kwargs)
 
     @Alias('torrents_topPrio')
@@ -969,13 +1019,12 @@ class TorrentsAPIMixIn(Request):
         """
         Set torrent as highest priority. Torrent Queuing must be enabled. (alias: torrents_topPrio)
 
-        Exceptions:
-            Conflict409
+        :raises Conflict409:
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='topPrio', data=data, **kwargs)
 
     @Alias('torrents_bottomPrio')
@@ -984,13 +1033,12 @@ class TorrentsAPIMixIn(Request):
         """
         Set torrent as highest priority. Torrent Queuing must be enabled. (alias: torrents_bottomPrio)
 
-        Exceptions:
-            Conflict409
+        :raises Conflict409:
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='bottomPrio', data=data, **kwargs)
 
     @Alias('torrents_downloadLimit')
@@ -1002,7 +1050,7 @@ class TorrentsAPIMixIn(Request):
 
         :return: dictioanry {hash: limit} (-1 represents no limit)
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         return self._post(_name=APINames.Torrents, _method='downloadLimit', data=data, **kwargs)
 
     @Alias('torrents_setDownloadLimit')
@@ -1015,7 +1063,7 @@ class TorrentsAPIMixIn(Request):
         :param limit: bytes/second (-1 sets the limit to infinity)
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'limit': limit}
         self._post(_name=APINames.Torrents, _method='setDownloadLimit', data=data, **kwargs)
 
@@ -1031,7 +1079,7 @@ class TorrentsAPIMixIn(Request):
         :param seeding_time_limit: minutes (-2 means use the global value and -1 is no limit)
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'ratioLimit': ratio_limit,
                 'seedingTimeLimit': seeding_time_limit}
         self._post(_name=APINames.Torrents, _method='setShareLimits', data=data, **kwargs)
@@ -1046,7 +1094,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: dictionary of limits
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         return self._post(_name=APINames.Torrents, _method='uploadLimit', data=data, **kwargs)
 
     @Alias('torrents_setUploadLimit')
@@ -1059,7 +1107,7 @@ class TorrentsAPIMixIn(Request):
         :param limit: bytes/second (-1 sets the limit to infinity)
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'limit': limit}
         self._post(_name=APINames.Torrents, _method='setUploadLimit', data=data, **kwargs)
 
@@ -1069,15 +1117,14 @@ class TorrentsAPIMixIn(Request):
         """
         Set location for torrents's files. (alias: torrents_setLocation)
 
-        Exceptions:
-            Forbidden403Error if the user doesn't have permissions to write to the location
-            Conflict409 if the directory cannot be created at the location
+        :raises Forbidden403Error: if the user doesn't have permissions to write to the location
+        :raises Conflict409: if the directory cannot be created at the location
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :param location: disk location to move torrent's files
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'location': location}
         self._post(_name=APINames.Torrents, _method='setLocation', data=data, **kwargs)
 
@@ -1087,14 +1134,13 @@ class TorrentsAPIMixIn(Request):
         """
         Set a category for one or more torrents. (alias: torrents_setCategory)
 
-        Exceptions:
-            Conflict409 for bad category
+        :raises Conflict409: for bad category
 
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :param category: category to assign to torrent
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'category': category}
         self._post(_name=APINames.Torrents, _method='setCategory', data=data, **kwargs)
 
@@ -1108,7 +1154,7 @@ class TorrentsAPIMixIn(Request):
         :param enable: True or False
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'enable': enable}
         self._post(_name=APINames.Torrents, _method='setAutoManagement', data=data, **kwargs)
 
@@ -1121,7 +1167,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'))}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'))}
         self._post(_name=APINames.Torrents, _method='toggleSequentialDownload', data=data, **kwargs)
 
     @Alias('torrents_toggleFirstLastPiecePrio')
@@ -1133,7 +1179,7 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|')}
         self._post(_name=APINames.Torrents, _method='toggleFirstLastPiecePrio', data=data, **kwargs)
 
     @Alias('torrents_setForceStart')
@@ -1146,7 +1192,7 @@ class TorrentsAPIMixIn(Request):
         :param enable: True or False (False makes this equivalent to torrents_resume())
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'value': enable}
         self._post(_name=APINames.Torrents, _method='setForceStart', data=data, **kwargs)
 
@@ -1160,7 +1206,7 @@ class TorrentsAPIMixIn(Request):
         :param enable: True or False
         :return:
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
                 'value': enable}
         self._post(_name=APINames.Torrents, _method='setSuperSeeding', data=data, **kwargs)
 
@@ -1172,15 +1218,14 @@ class TorrentsAPIMixIn(Request):
         """
         Add one or more peers to one or more torrents. (alias: torrents_addPeers)
 
-        Exceptions:
-            InvalidRequest400Error for invalid peers
+        :raises InvalidRequest400Error: for invalid peers
 
         :param peers: one or more peers to add. each peer should take the form 'host:port'
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: dictionary - {<hash>: {'added': #, 'failed': #}}
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
-                'peers': list2string(peers, '|')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+                'peers': self._list2string(peers, '|')}
         return self._post(_name=APINames.Torrents, _method='addPeers', data=data, **kwargs)
 
     # TORRENT CATEGORIES ENDPOINTS
@@ -1205,8 +1250,7 @@ class TorrentsAPIMixIn(Request):
 
         Note: save_path is not available until web API version 2.1.0
 
-        Exceptions:
-            Conflict409 if category name is not valid or unable to create
+        :raises Conflict409: if category name is not valid or unable to create
 
         :param name: name for new category
         :param save_path: location to save torrents for this category
@@ -1225,8 +1269,7 @@ class TorrentsAPIMixIn(Request):
 
         Note: torrents/editCategory not available until web API version 2.1.0
 
-        Exceptions:
-            Conflict409
+        :raises Conflict409:
 
         :param name: category to edit
         :param save_path: new location to save files for this category
@@ -1245,7 +1288,7 @@ class TorrentsAPIMixIn(Request):
         :param categories: categories to delete
         :return: None
         """
-        data = {'categories': list2string(categories, '\n')}
+        data = {'categories': self._list2string(categories, '\n')}
         self._post(_name=APINames.Torrents, _method='removeCategories', data=data, **kwargs)
 
     # TORRENT TAGS ENDPOINTS
@@ -1272,8 +1315,8 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
-                'tags': list2string(tags, ',')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+                'tags': self._list2string(tags, ',')}
         self._post(_name=APINames.Torrents, _method='addTags', data=data, **kwargs)
 
     @Alias('torrents_removeTags')
@@ -1287,8 +1330,8 @@ class TorrentsAPIMixIn(Request):
         :param torrent_hashes: single torrent hash or list of torrent hashes. Or 'all' for all torrents.
         :return: None
         """
-        data = {'hashes': list2string(torrent_hashes or kwargs.get('hashes'), '|'),
-                'tags': list2string(tags, ',')}
+        data = {'hashes': self._list2string(torrent_hashes or kwargs.get('hashes'), '|'),
+                'tags': self._list2string(tags, ',')}
         self._post(_name=APINames.Torrents, _method='removeTags', data=data, **kwargs)
 
     @Alias('torrents_createTags')
@@ -1301,7 +1344,7 @@ class TorrentsAPIMixIn(Request):
         :param tags: tag name or list of tags
         :return: None
         """
-        data = {'tags': list2string(tags, ',')}
+        data = {'tags': self._list2string(tags, ',')}
         self._post(_name=APINames.Torrents, _method='createTags', data=data, **kwargs)
 
     @Alias('torrents_deleteTags')
@@ -1314,5 +1357,5 @@ class TorrentsAPIMixIn(Request):
         :param tags: tag name or list of tags
         :return: None
         """
-        data = {'tags': list2string(tags, ',')}
+        data = {'tags': self._list2string(tags, ',')}
         self._post(_name=APINames.Torrents, _method='deleteTags', data=data, **kwargs)
